import takeoff_config
import pytest
import yaml

CORRECT_PATH = "test.yaml"
CORRECT_READER = "reader1"
with open(CORRECT_PATH, "r") as file:
    data = yaml.safe_load(file)
    CORRECT_READER_BODY = data["takeoff"]["readers_config"][CORRECT_READER]


def test_reader_returns_correct_reader():
    reader = takeoff_config.read_takeoff_readers_config(CORRECT_PATH, CORRECT_READER)
    for key, val in CORRECT_READER_BODY.items():
        assert reader.__getattribute__(key) == val


def test_reader_fails_if_given_wrong_path():
    path = "not-path"
    try:
        takeoff_config.read_takeoff_readers_config(path, CORRECT_READER)
    except Exception as e:
        assert f"File Not Found on path: {path}" in str(e)


def test_reader_fails_if_given_wrong_reader():
    reader = "not-reader"
    try:
        takeoff_config.read_takeoff_readers_config(CORRECT_PATH, reader)
    except Exception as e:
        assert f"No reader with id {reader} found in manifest.yaml" in str(e)


def testing_dict_conversion_of_reader():
    reader = takeoff_config.read_takeoff_readers_config(CORRECT_PATH, CORRECT_READER)
    reader_dict = reader.dict_without_optionals()
    with pytest.raises(KeyError):
        reader_dict["cuda_visible_devices"]
    for key, value in reader_dict.items():
        assert CORRECT_READER_BODY[key] == value



from typing import Optional

class AppConfig:
    max_batch_size: int
    batch_duration_millis: int
    echo: bool
    port: int
    enable_metrics: bool
    heartbeat_check_interval: int
    launch_management_server: bool
    launch_vertex_server: bool
    launch_sagemaker_server: bool
    management_port: int
    vertex_port: int

class ReaderConfig:
    model_name: str
    device: str
    consumer_group: str
    redis_host: Optional[str]
    backend: Optional[str]
    log_level: Optional[str]
    cuda_visible_devices: Optional[str]
    disable_continuous_generation: Optional[bool]
    max_batch_size: Optional[int]

    def dict_without_optionals(self) -> dict[str, str | int | bool]:
        """Creates Dictionary from ReaderConfig, with optional values removed.
        Returns:
            dict[str, Optional[str | int]]: Dict without optionals.
        """
        ...

def read_takeoff_readers_config(path: str, reader_id: str) -> ReaderConfig:
    """Fetches ReaderConfig from a yaml on the given path and reader_id.
    Returns:
        ReaderConfig: ReaderConfig object corresponding to the given reader_id.
    """
    ...



"""This script is used to bump the version of the package. Cargo.toml is used to store the version."""
import toml

def read_version():
    with open("./Cargo.toml", "r") as file:
        data = toml.load(file)
    return data["package"]["version"]

def read_takeoff_client_version():
    with open("../takeoff-client/python-client/pyproject.toml", "r") as file:
        data = toml.load(file)
    return data["project"]["version"]

def write_version(new_version):
    # Read the content of the file
    with open("./Cargo.toml", "r") as file:
        content = file.readlines()

    # Find and update the version line
    for i, line in enumerate(content):
        if line.startswith("version = "):
            content[i] = f'version = "{new_version}"\n'
            break

    # Write the updated content back to the file
    with open("./Cargo.toml", "w") as file:
        file.writelines(content)
    
    # Update version takeoff-gateway version uses
    # Read the content of the file
    with open("../takeoff-gateway/Cargo.toml", "r") as file:
        content = toml.load(file)

    # Find and update the takeoff-config line
    content["dependencies"]["takeoff-config"] = { "version": new_version, "path": "../takeoff-config" }

    # Write the updated content back to the file
    with open("../takeoff-gateway/Cargo.toml", "w") as file:
        toml.dump(content, file)


if __name__ == "__main__":
    new_version = read_takeoff_client_version()
    write_version(new_version)
    print(f"Version updated to: {new_version}")
